@@ -1,6 +1,7 @@
 # Copyright (C) 1997-2001 Damian Conway.  All rights reserved.
 # Copyright (C) 2009 Adam Kennedy.
-# Copyright (C) 2015 Steve Hay.  All rights reserved.
+# Copyright (C) 2015, 2022 Steve Hay and other contributors.  All rights
+# reserved.
 
 # This module is free software; you can redistribute it and/or modify it under
 # the same terms as Perl itself, i.e. under the terms of either the GNU General
@@ -17,7 +18,7 @@ use Exporter ();
 
 use vars qw { $VERSION @ISA %EXPORT_TAGS };
 BEGIN {
-    $VERSION     = '2.04_02';
+    $VERSION     = '2.05_01';
     @ISA         = 'Exporter';
     %EXPORT_TAGS = (
         ALL => [ qw{
@@ -40,6 +41,7 @@ Exporter::export_ok_tags('ALL');
 our $RE_PREREGEX_PAT = qr#(
     [!=]~
     | split|grep|map
+    | not|and|or|xor
 )#x;
 our $RE_EXPR_PAT = qr#(
     (?:\*\*|&&|\|\||<<|>>|//|[-+*x%^&|.])=?
@@ -492,7 +494,7 @@ sub _match_variable
         my $deref = $1;
 
         unless ($$textref =~ m/\G\s*(?:::|')?(?:[_a-z]\w*(?:::|'))*[_a-z]\w*/gci
-            or _match_codeblock($textref, qr/\G()/, '\{', qr/\G\s*(\})/, '\{', '\}', 0)
+            or _match_codeblock($textref, qr/\G()/, '\{', qr/\G\s*(\})/, '\{', '\}', 0, 1)
             or $deref eq '$#' or $deref eq '$$'
             or pos($$textref) == length $$textref )
         {
@@ -508,10 +510,10 @@ sub _match_variable
         next if _match_codeblock($textref,
                                  qr/\G\s*->\s*(?:[_a-zA-Z]\w+\s*)?/,
                                  qr/[({[]/, qr/\G\s*([)}\]])/,
-                                 qr/[({[]/, qr/[)}\]]/, 0);
+                                 qr/[({[]/, qr/[)}\]]/, 0, 1);
         next if _match_codeblock($textref,
                                  qr/\G\s*/, qr/[{[]/, qr/\G\s*([}\]])/,
-                                 qr/[{[]/, qr/[}\]]/, 0);
+                                 qr/[{[]/, qr/[}\]]/, 0, 1);
         next if _match_variable($textref,qr/\G\s*->\s*/);
         next if $$textref =~ m/\G\s*->\s*\w+(?![{([])/gc;
         last;
@@ -555,7 +557,7 @@ sub extract_codeblock (;$$$$$)
     my $rd         = $_[4];
     my @delims = _ec_delims($ldel_inner, $ldel_outer);
 
-    my @match = _match_codeblock($textref, $pre, @delims, $rd);
+    my @match = _match_codeblock($textref, $pre, @delims, $rd, 1);
     return _fail($wantarray, $textref) unless @match;
     return _succeed($wantarray, $textref,
                     @match[2..3,4..5,0..1]    # MATCH, REMAINDER, PREFIX
@@ -564,7 +566,8 @@ sub extract_codeblock (;$$$$$)
 
 sub _match_codeblock
 {
-    my ($textref, $pre, $ldel_outer, $rdel_outer, $ldel_inner, $rdel_inner, $rd) = @_;
+    my ($textref, $pre, $ldel_outer, $rdel_outer, $ldel_inner, $rdel_inner, $rd, $no_backcompat) = @_;
+    $rdel_outer = qr/\G\s*($rdel_outer)/ if !$no_backcompat; # Switch calls this func directly
     my $startpos = pos($$textref) = pos($$textref) || 0;
     unless ($$textref =~ m/$pre/gc)
     {
@@ -628,7 +631,7 @@ sub _match_codeblock
             next;
         }
 
-        if ( _match_codeblock($textref, qr/\G\s*/, $ldel_inner, qr/\G\s*($rdel_inner)/, $ldel_inner, $rdel_inner, $rd) )
+        if ( _match_codeblock($textref, qr/\G\s*/, $ldel_inner, qr/\G\s*($rdel_inner)/, $ldel_inner, $rdel_inner, $rd, 1) )
         {
             $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 1;
             next;
@@ -655,6 +658,7 @@ sub _match_codeblock
         return;
     }
 
+    $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = undef;
     my $endpos = pos($$textref);
     return ( $startpos, $codepos-$startpos,
              $codepos, $endpos-$codepos,
@@ -889,7 +893,7 @@ sub _match_quotelike
 
     $$textref =~ m/\G($mods{$op})/gc;
     my $endpos = pos $$textref;
-    $ref2qmarkvalid{$textref} = $ref2slashvalid{$textref} = 0;
+    $ref2qmarkvalid{$textref} = $ref2slashvalid{$textref} = undef;
 
     return (
         $startpos,      $oppos-$startpos,       # PREFIX
@@ -2406,7 +2410,8 @@ Copyright (C) 1997-2001 Damian Conway.  All rights reserved.
 
 Copyright (C) 2009 Adam Kennedy.
 
-Copyright (C) 2015, 2020 Steve Hay.  All rights reserved.
+Copyright (C) 2015, 2020, 2022 Steve Hay and other contributors.  All rights
+reserved.
 
 =head1 LICENCE
 
@@ -2416,7 +2421,7 @@ License or the Artistic License, as specified in the F<LICENCE> file.
 
 =head1 VERSION
 
-Version 2.05
+Version 2.06
 
 =head1 DATE
 
